c=======================================================================
c=====|==|=========|=========|=========|=========|=========|=========|==
c
      program p1_CMORPH_V10_CRT_025deg_DLY_00Z_def_percentile
      implicit none

c-----------------------------------------------------------------------
c NX  : X-dimension (0-360) in 0.25deg-grid
c NY  : Y-dimension (60S-60N) in 0.25deg-grid
c NV  : number of variables in output file
c NK  : max number of 0.25deg-DLY data points monthly (20 years * 31 days)
c NK9 : NK * 9 (nearby grid boxes)
c-----------------------------------------------------------------------
      integer NX, NY, NV, NK, NK9
      parameter( NX=1440, NY=480, NV=12, NK=20*31, NK9=NK*9 )

      character f_msk*55, f_dly*58, f_pct*48
      integer yyyy, mm, dd, days(12), kk, k, kct
      integer ii, i1, i2, i, is, jj, j1, j2, j, js
      integer k99, k98, k97, k96, k95, k94, k93, k92, k91, k90
      real msk(NX,NY), vv(NX,NY), clim(NX,NY), vvk(NX,NY,NK), val(NK9)
      real p99(NX,NY), p98(NX,NY), p97(NX,NY), p96(NX,NY), p95(NX,NY)
      real p94(NX,NY), p93(NX,NY), p92(NX,NY), p91(NX,NY), p90(NX,NY)
      integer ct(NX,NY), cr(NX,NY), ck(NK9)
      real rpct(NX,NY)

      data days / 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 /

c-----------------------------------------------------------------------
c construct file names
c-----------------------------------------------------------------------
c----------->123456789|123456789|123456789|123456789|123456789|12345678<--
      f_msk='../../../system/table/CMORPH_V1.0_LAND_MASK_0.25deg.lnx'
      f_dly='../../y???/y???m?/CMORPH_V1.0_ADJ_0.25deg-DLY_00Z_y???m?d?'
      f_pct='../CMORPH_V1.0_ADJ_0.25deg-DLY_00Z_PCT_1998-2017'

c-----------------------------------------------------------------------
c read in land mask
c-----------------------------------------------------------------------
      open(10, file=f_msk, status='old', access='direct', recl=NX*NY*4)
      read(10, rec=1) msk
      close(10)

c-----------------------------------------------------------------------
c open output file
c-----------------------------------------------------------------------
      open(20, file=f_pct, access='direct', recl=NX*NY*4)

c-----------------------------------------------------------------------
c repeat for each month
c-----------------------------------------------------------------------
      do mm=1, 12

        kk=0

c-----------------------------------------------------------------------
c read in all 0.25deg-DLY data in target month
c-----------------------------------------------------------------------
        do yyyy=1998, 2017
          write(f_dly( 7:10), '(i4.4)') yyyy
          write(f_dly(12:17), '(i4.4,i2.2)') yyyy, mm
          days(2)=28
          if( mod(yyyy,4).eq.0 ) days(2)=29
          do dd=1, days(mm)
            kk=kk+1
            write(f_dly(51:58), '(i4.4,2i2.2)') yyyy, mm, dd
            open(10, file=f_dly, status='old',
     1          access='direct', recl=NX*NY*4)
            write(*, *) ' <--  ', f_dly
            read(10, rec=1) vv
            close(10)
            do jj=1, NY
              do ii=1, NX
                vvk(ii,jj,kk)=vv(ii,jj)
              enddo
            enddo
          enddo
        enddo

        kct=kk
        write(*, *) mm, kct

c-----------------------------------------------------------------------
c define climo and percentage of rainy
c-----------------------------------------------------------------------
        do jj=1, NY
          do ii=1, NX
            ct(ii,jj)=0
            cr(ii,jj)=0
            clim(ii,jj)=0.0
          enddo
        enddo
        do jj=1, NY
          do ii=1, NX
            do kk=1, kct
              if( vvk(ii,jj,kk).ge.0.0 ) then
                ct(ii,jj)=ct(ii,jj)+1
                clim(ii,jj)=clim(ii,jj)+vvk(ii,jj,kk)
              endif
              if( vvk(ii,jj,kk).ge.1.0 ) cr(ii,jj)=cr(ii,jj)+1
            enddo
          enddo
        enddo
        do jj=1, NY
          do ii=1, NX
            if( ct(ii,jj).eq.0 ) then
              clim(ii,jj)=-999.0
              rpct(ii,jj)=-999.0
            else
              clim(ii,jj)=clim(ii,jj)/ct(ii,jj)
              rpct(ii,jj)=1.0*cr(ii,jj)/ct(ii,jj)
            endif
          enddo
        enddo

c-----------------------------------------------------------------------
c define percentiles (including nearby grid boxes)
c-----------------------------------------------------------------------
        do jj=1, NY

          write(*, *) jj

          j1=jj-1
          j2=jj+1
          if( j1.lt.1 )  j1=1
          if( j2.gt.NY ) j2=NY

          do ii=1, NX

            i1=ii-1
            i2=ii+1

            k=0

            do j=j1, j2
              js=j
              do i=i1, i2
                if( i.lt.1 ) then
                  is=i+NX
                elseif( i.gt.NX ) then
                  is=i-NX
                else
                  is=i
                endif

c-----------------------------------------------------------------------
c only use nearby grid boxes with same surface type
c-----------------------------------------------------------------------
                if( msk(is,js).ne.msk(ii,jj) ) cycle

                do kk=1, kct
                  k=k+1
                  val(k)=vvk(is,js,kk)
                enddo

              enddo
            enddo

            call qsort(val, k, ck)

            k99=0.99*k
            k98=0.98*k
            k97=0.97*k
            k96=0.96*k
            k95=0.93*k
            k94=0.94*k
            k93=0.93*k
            k92=0.92*k
            k91=0.91*k
            k90=0.90*k

            p99(ii,jj)=val(k99)
            p98(ii,jj)=val(k98)
            p97(ii,jj)=val(k97)
            p96(ii,jj)=val(k96)
            p95(ii,jj)=val(k95)
            p94(ii,jj)=val(k94)
            p93(ii,jj)=val(k93)
            p92(ii,jj)=val(k92)
            p91(ii,jj)=val(k91)
            p90(ii,jj)=val(k90)

          enddo
        enddo

c-----------------------------------------------------------------------
c output
c-----------------------------------------------------------------------
        write(20, rec=(mm-1)*NV+1)  clim
        write(20, rec=(mm-1)*NV+2)  rpct
        write(20, rec=(mm-1)*NV+3)  p99
        write(20, rec=(mm-1)*NV+4)  p98
        write(20, rec=(mm-1)*NV+5)  p97
        write(20, rec=(mm-1)*NV+6)  p96
        write(20, rec=(mm-1)*NV+7)  p95
        write(20, rec=(mm-1)*NV+8)  p94
        write(20, rec=(mm-1)*NV+9)  p93
        write(20, rec=(mm-1)*NV+10) p92
        write(20, rec=(mm-1)*NV+11) p91
        write(20, rec=(mm-1)*NV+12) p90

      enddo
      close(20)

      stop
      end
c
c=====|==|=========|=========|=========|=========|=========|=========|==
c
c
c     #######################################################################
c     ###                                                                 ###
c     ###  Subroutine  :                                                  ###
c     ###                                                                 ###
c     ###                 name:       qsortchr_PPX(arr,n, karr)           ###
c     ###                                                                 ###
c     ###                 function:   to sort an array in accesending     ###
c     ###                             orider                              ###
c     ###                                                                 ###
c     ###                 inputs:                                         ###
c     ###                             arr  ==   the array to be sorted    ###
c     ###                             n    ==   dimension of the array    ###
c     ###                                                                 ###
c     ###                 output:                                         ###
c     ###                             karr ==   position of the sorted    ###
c     ###                             arr  ==   the sorted array          ###
c     ###                                                                 ###
c     #######################################################################
c
C  qsortReal: sort real array elements
c (Numerical Recioes in FORTRAN 77.  2nd edition. Page 324-325.)
C
      subroutine qsort(arr, n, karr)
c
      integer*4 n,M,NSTACK
      real*4 arr(n)
      parameter(M=7,NSTACK=1000000)
      integer*4 i,ir,j,jstack,k,l,istack(NSTACK)
      real*4 a,temp
      integer*4  karr (n)

      do i = 1, n
        karr (i) = i
      end  do

      jstack=0
      L=1
      ir=n


   1  if(ir-L.lt.M) then
       do j=L+1,ir
        a=arr(j)
        ka = karr(j)
        do i=j-1,L,-1
         if(arr(i).lt.a) goto 2
         arr(i+1)=arr(i)
         karr(i+1) = karr(i)
        enddo
        i=L-1
   2    arr(i+1)=a
        karr(i+1) = ka
       enddo
       if(jstack.eq.0) return
       ir=istack(jstack)
       L=istack(jstack-1)
       jstack=jstack-2
      else
       k=(L+ir)/2
       temp=arr(k)
       ktemp = karr(k)
       arr(k)=arr(L+1)
       karr(k)=karr(L+1)
       arr(L+1)=temp
       karr(L+1)=ktemp
       if(arr(L).gt.arr(ir)) then
        temp=arr(L)
        ktemp=karr(L)
        arr(L)=arr(ir)
        karr(L) = karr(ir)
        arr(ir)=temp
        karr(ir)=ktemp
       endif
       if(arr(L+1).gt.arr(ir)) then
        temp=arr(L+1)
        ktemp = karr(L+1)
        arr(L+1)=arr(ir)
        karr(L+1) = karr(ir)
        arr(ir)=temp
        karr(ir) = ktemp
       endif
       if(arr(L).gt.arr(L+1)) then
        temp=arr(L)
        ktemp = karr(L)
        arr(L)=arr(L+1)
        karr(L) = karr(L+1)
        arr(L+1)=temp
        karr(L+1)=ktemp
       endif
       i=L+1
       j=ir
       a=arr(L+1)
       ka = karr(L+1)
   3   continue
       i=i+1
       if(arr(i).lt.a) goto 3
   4   continue
       j=j-1
       if(arr(j).gt.a) goto 4
       if(j.lt.i) goto 5
       temp=arr(i)
       ktemp = karr(i)
       arr(i)=arr(j)
       karr(i) = karr(j)
       arr(j)=temp
       karr(j)= ktemp
       goto 3
   5   arr(L+1)=arr(j)
       karr(L+1)=karr(j)
       arr(j)=a
       karr(j) = ka
       jstack=jstack+2
c      if(jstack.gt.NSTACK) pause 'NSTACK too small in sort'
       if(jstack.gt.NSTACK) stop 'NSTACK too small in sort'
       if(ir-i+1.ge.j-L) then
        istack(jstack)=ir
        istack(jstack-1)=i
        ir=j-1
       else
        istack(jstack)=j-1
        istack(jstack-1)=L
        L=i
       endif
      endif
      goto 1
      return
      end
